/*
 * SourceMod Radio
 *
 * Adds a radio player to SourceMod that supports direct station links or streaming audio
 *
 * Coded by dubbeh - www.dubbeh.net
 *
 * Licensed under the GPLv3
 *
 */


int GetAvailableWebSocketLibrary()
{
	// Prioritise using SteamWorks
	if (STEAMWORKS_AVAILABLE())
	{
		LogMessage("[SM-RADIO] SteamWorks extension found.");
		return WEBSOCKET_TYPE_STEAMWORKS;
	}
	else if (CURL_AVAILABLE())
	{
		LogMessage("[SM-RADIO] CURL extension found.");
		return WEBSOCKET_TYPE_CURL;
	}
	else if (SOCKET_AVAILABLE())
	{
		LogMessage("[SM-RADIO] Sockets extension found but unsupported. Please use SteamWorks or CURL.");
		return WEBSOCKET_TYPE_SOCKETS;
	}
	
	return -1;
}

void UpdateShoutcastStationsList()
{
	if (g_iWebSocketType == WEBSOCKET_TYPE_CURL) {
		UpdateUsingCURL();
	} else if (g_iWebSocketType == WEBSOCKET_TYPE_STEAMWORKS) {
		UpdateUsingSteamWorks();
	} else if (g_iWebSocketType == WEBSOCKET_TYPE_SOCKETS) {
		UpdateUsingSockets();
	}
}

/* Updater using CURL API functions */
void UpdateUsingCURL()
{
	Handle hConnection = INVALID_HANDLE;
	char szLocalPath[PLATFORM_MAX_PATH + 1];
	Handle hUpdateCheckTempFile = INVALID_HANDLE;
	
	hConnection = curl_easy_init();
	if (hConnection != INVALID_HANDLE)
	{
		BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szPlayListVersionTempFile);
		hUpdateCheckTempFile = curl_OpenFile(szLocalPath, "w");
		CURL_DEFAULT_OPT(hConnection);
		curl_easy_setopt_handle(hConnection, CURLOPT_WRITEDATA, hUpdateCheckTempFile);
		curl_easy_setopt_string(hConnection, CURLOPT_URL, g_szShoutCastVersionURL);
		curl_easy_perform_thread(hConnection, CURL_OnVersionCheckComplete, hUpdateCheckTempFile);
	}
}

public void CURL_OnVersionCheckComplete(Handle hndl, CURLcode code, any hUpdateCheckTempFile)
{
	char szLocalPath[PLATFORM_MAX_PATH + 1];
	File hFile = null;
	Handle hConnection = INVALID_HANDLE;
	Handle hUpdatedPlaylistFile = INVALID_HANDLE;
	int iFileSize = 0;
	char szTempBuffer[64];
	int iOnlinePlaylistVersion = 0;
	
	// Free up the connection and file handle(s)
	if (hndl != INVALID_HANDLE)
		CloseHandle(hndl);
	if (hUpdateCheckTempFile != INVALID_HANDLE)
		CloseHandle(hUpdateCheckTempFile);
	
	if (code == CURLE_OK)
	{
		// Lets open the temp version file
		BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szPlayListVersionTempFile);
		iFileSize = FileSize(szLocalPath);
		if (iFileSize != -1 && iFileSize > 0)
		{
			hFile = OpenFile(szLocalPath, "r");
			if (hFile != INVALID_HANDLE)
			{
				hFile.ReadLine(szTempBuffer, sizeof(szTempBuffer));
				hFile.Close();
				DeleteFile(szLocalPath);
				
				iOnlinePlaylistVersion = StringToInt(szTempBuffer);
				// Check current playlist against web playlist version
				if (StringToInt(szTempBuffer) > g_iShoutCastPlaylistVersion)
				{
					LogMessage("[SM-RADIO] Found a newer playlist online - Online version \"%d\" - Local version \"%d\". Updating local version now.", iOnlinePlaylistVersion, g_iShoutCastPlaylistVersion);
					
					// Assume we need to update the playlist
					hConnection = curl_easy_init();
					if (hConnection != INVALID_HANDLE)
					{
						BuildPath(Path_SM, szLocalPath, sizeof(szLocalPath), "configs/%s", g_szPlayListTempFile);
						hUpdatedPlaylistFile = curl_OpenFile(szLocalPath, "w");
						CURL_DEFAULT_OPT(hConnection);
						curl_easy_setopt_handle(hConnection, CURLOPT_WRITEDATA, hUpdatedPlaylistFile);
						curl_easy_setopt_string(hConnection, CURLOPT_URL, g_szShoutCastPlaylistURL);
						curl_easy_perform_thread(hConnection, CURL_OnPlaylistDownloadComplete, hUpdatedPlaylistFile);
					}
				}
			}
		} else {
			LogMessage("[SM-RADIO] Unable to find the temp playlist version file or corrupted (0 bytes in size).");
		}
	} else {
		LogMessage("[SM-RADIO] Error checking the online ShoutCast playlist version returned \"%d\". Maybe the website is down?", code);
	}
}

public void CURL_OnPlaylistDownloadComplete(Handle hndl, CURLcode code, any hPlaylistFile)
{
	char szTempPlaylist[PLATFORM_MAX_PATH + 1];
	char szPlaylist[PLATFORM_MAX_PATH + 1];
	
	// Free up the connection and file handle(s)
	if (hndl != INVALID_HANDLE)
		CloseHandle(hndl);
	if (hPlaylistFile != INVALID_HANDLE)
		CloseHandle(hPlaylistFile);
	
	if (code == CURLE_OK)
	{
		// Overwrite the old playlist file with the new one
		BuildPath(Path_SM, szPlaylist, sizeof(szPlaylist), "configs/%s", g_szRadioStationsShoutCastBaseFile);
		DeleteFile(szPlaylist);
		
		BuildPath(Path_SM, szTempPlaylist, sizeof(szTempPlaylist), "configs/%s", g_szPlayListTempFile);
		RenameFile(szPlaylist, szTempPlaylist);
		
		CPrintToChatAll("[SM-RADIO] ShoutCast station list updated. Please reload the map to apply changes.");
		LogMessage("[SM-RADIO] ShoutCast station list updated. Please reload the map to apply changes.");
	} else {
		LogMessage("[SM-RADIO] Error downloading the latest ShoutCast playlist, CURL returned \"%d\". Maybe the website is down?", code);
	}
}

/*
 * Updater using SteamWorks API Functions
 *
 * Thanks to Kyle Sanderson for example code
 */

void UpdateUsingSteamWorks()
{
	Handle hHTTPRequest = INVALID_HANDLE;
	
	if (SteamWorks_IsLoaded())
	{
		if ((hHTTPRequest = SteamWorks_CreateHTTPRequest(k_EHTTPMethodGET, g_szShoutCastVersionURL)) != INVALID_HANDLE)
		{
			if (SteamWorks_SetHTTPCallbacks(hHTTPRequest, SteamWorks_OnVersionCheckComplete) && SteamWorks_SendHTTPRequest(hHTTPRequest))
			{
				LogMessage("[SM-RADIO] Running HTTP request using SteamWorks for URL=\"%s\"", g_szShoutCastVersionURL);
			}
			else
			{
				CloseHandle(hHTTPRequest);
				LogMessage("[SM-RADIO] Error: UpdateUsingSteamWorks() Problem sending HTTP request.");
			}
		} else {
			LogMessage("[SM-RADIO] Error creating HTTP request using SteamWorks for URL=\"%s\"", g_szShoutCastVersionURL);
		}
	} else {
		LogMessage("[SM-RADIO] Error querying SteamWorks_IsLoaded(). Please check the plugin is installed correctly and functioning.");
	}
	
	return;
}

public void SteamWorks_OnVersionCheckComplete(Handle hRequest, bool bFailure, bool bRequestSuccessful, EHTTPStatusCode eStatusCode)
{
	if (!bFailure && bRequestSuccessful && eStatusCode == k_EHTTPStatusCode200OK)
	{
		LogMessage("[SM-RADIO] SteamWorks_OnVersionCheckComplete() Success");
		SteamWorks_GetHTTPResponseBodyCallback(hRequest, SteamWorks_VersionAPIWebResponse);
	} else {
		LogMessage("[SM-RADIO] SteamWorks_OnVersionCheckComplete() Error: Response code %d", eStatusCode);
	}
	
	CloseHandle(hRequest);
}

public void SteamWorks_VersionAPIWebResponse(const char[] szResponseBuffer)
{
	int iOnlinePlaylistVersion = 0;
	Handle hHTTPRequest = INVALID_HANDLE;
	
	// Check current playlist against web playlist version
	iOnlinePlaylistVersion = StringToInt(szResponseBuffer);
	LogMessage("[SM-RADIO] SteamWorks_VersionAPIWebResponse() Checking online version reported as %d - Local version %d", iOnlinePlaylistVersion, g_iShoutCastPlaylistVersion);
	
	if (iOnlinePlaylistVersion > g_iShoutCastPlaylistVersion)
	{
		if ((hHTTPRequest = SteamWorks_CreateHTTPRequest(k_EHTTPMethodGET, g_szShoutCastPlaylistURL)) != INVALID_HANDLE)
		{
			if (SteamWorks_SetHTTPCallbacks(hHTTPRequest, SteamWorks_OnPlaylistTransferComplete) && SteamWorks_SendHTTPRequest(hHTTPRequest))
			{
				g_iShoutCastPlaylistVersion = iOnlinePlaylistVersion;
			} else {
				CloseHandle(hHTTPRequest);
				LogMessage("[SM-RADIO] Error creating HTTP callbacks using SteamWorks.");
			}
		} else {
			LogMessage("[SM-RADIO] Error creating HTTP request using SteamWorks for URL=\"%s\"", g_szShoutCastVersionURL);
		}
	} else {
		LogMessage("[SM-RADIO] Online playlist matches local one. No need to update.");
	}
}

public void SteamWorks_OnPlaylistTransferComplete(Handle hRequest, bool bFailure, bool bRequestSuccessful, EHTTPStatusCode eStatusCode)
{
	if (!bFailure && bRequestSuccessful && eStatusCode == k_EHTTPStatusCode200OK)
	{
		SteamWorks_GetHTTPResponseBodyCallback(hRequest, SteamWorks_PlaylistAPIWebResponse);
	}
	
	CloseHandle(hRequest);
}

public void SteamWorks_PlaylistAPIWebResponse(const char[] szResponseBuffer)
{
	char szPlaylistPath[PLATFORM_MAX_PATH + 1] = "";
	File hPlaylistFile = null;
	
	BuildPath(Path_SM, szPlaylistPath, sizeof(szPlaylistPath), "configs/%s", g_szRadioStationsShoutCastBaseFile);
	
	// Overwrite current playlist with the new one
	hPlaylistFile = OpenFile(szPlaylistPath, "w");
	if (!hPlaylistFile.WriteString(szResponseBuffer, true))
	{
		LogMessage("[SM-RADIO] SteamWorks_PlaylistAPIWebResponse() Error writing response buffer to the new playlist.");
	} else {
		CPrintToChatAll("[SM-RADIO] ShoutCast station list updated. Please reload the map to apply changes.");
		LogMessage("[SM-RADIO] ShoutCast station list updated. Please reload the map to apply changes.");
	}
	
	CloseHandle(hPlaylistFile);
}

/* TODO: Updater using Sockets API Functions */
void UpdateUsingSockets()
{
	LogMessage("[SM-RADIO] Sockets currently unsupported. Use SteamWorks or CURL.");
	return;
}
